@@ -1,12 +1,13 @@
 use std::path::PathBuf;
 
-use crate::shader_module;
+use crate::shader_compiler::ShaderCompiler;
 
 use color_eyre::Result;
 use pilka_ash::{AshRender, HasRawWindowHandle};
-use pilka_types::{ContiniousHashMap, Frame, ImageDimentions, PipelineInfo, ShaderCreateInfo};
+use pilka_types::{
+    ContiniousHashMap, Frame, ImageDimentions, PipelineInfo, PushConstant, ShaderCreateInfo,
+};
 use pilka_wgpu::WgpuRender;
-use shaderc::Compiler;
 
 pub trait Renderer {
     fn get_info(&self) -> String;
@@ -15,7 +16,7 @@ pub trait Renderer {
 
     fn resize(&mut self, width: u32, height: u32) -> Result<()>;
 
-    fn render(&mut self, push_constant: &[u8]) -> Result<()>;
+    fn render(&mut self, push_constant: PushConstant) -> Result<()>;
 
     fn capture_frame(&mut self) -> Result<Frame>;
     fn captured_frame_dimentions(&self) -> ImageDimentions;
@@ -68,7 +69,7 @@ impl<'a> RenderBundleStatic<'a> {
         &mut self,
         pipeline: PipelineInfo,
         includes: &[PathBuf],
-        shader_compiler: &mut Compiler,
+        shader_compiler: &mut ShaderCompiler,
     ) -> Result<()> {
         puffin::profile_function!();
         let pipeline_number = self.pipelines.len();
@@ -79,19 +80,13 @@ impl<'a> RenderBundleStatic<'a> {
                 self.shader_set
                     .push_value(vert.path.canonicalize()?, pipeline_number);
 
-                let vert_artifact = shader_module::create_shader_module(
-                    vert,
-                    shaderc::ShaderKind::Vertex,
-                    shader_compiler,
-                )?;
-                let vert = ShaderCreateInfo::new(vert_artifact.as_binary(), &vert.entry_point);
+                let vert_artifact =
+                    shader_compiler.create_shader_module(vert, shaderc::ShaderKind::Vertex)?;
+                let vert = ShaderCreateInfo::new(&vert_artifact, &vert.entry_point);
 
-                let frag_arifact = shader_module::create_shader_module(
-                    frag,
-                    shaderc::ShaderKind::Fragment,
-                    shader_compiler,
-                )?;
-                let frag = ShaderCreateInfo::new(frag_arifact.as_binary(), &frag.entry_point);
+                let frag_arifact =
+                    shader_compiler.create_shader_module(frag, shaderc::ShaderKind::Fragment)?;
+                let frag = ShaderCreateInfo::new(&frag_arifact, &frag.entry_point);
 
                 match self.kind.as_mut().unwrap() {
                     Backend::Ash(ash) => ash.push_render_pipeline(vert, frag)?,
@@ -102,12 +97,9 @@ impl<'a> RenderBundleStatic<'a> {
                 self.shader_set
                     .push_value(comp.path.canonicalize()?, pipeline_number);
 
-                let comp_artifact = shader_module::create_shader_module(
-                    comp,
-                    shaderc::ShaderKind::Compute,
-                    shader_compiler,
-                )?;
-                let comp = ShaderCreateInfo::new(comp_artifact.as_binary(), &comp.entry_point);
+                let comp_artifact =
+                    shader_compiler.create_shader_module(comp, shaderc::ShaderKind::Compute)?;
+                let comp = ShaderCreateInfo::new(&comp_artifact, &comp.entry_point);
 
                 match self.kind.as_mut().unwrap() {
                     Backend::Ash(ash) => ash.push_compute_pipeline(comp)?,
@@ -128,7 +120,7 @@ impl<'a> RenderBundleStatic<'a> {
     pub fn register_shader_change(
         &mut self,
         paths: &[PathBuf],
-        shader_compiler: &mut Compiler,
+        shader_compiler: &mut ShaderCompiler,
     ) -> Result<()> {
         puffin::profile_function!();
         self.wait_idle();
@@ -137,21 +129,13 @@ impl<'a> RenderBundleStatic<'a> {
                 for &index in pipeline_indices {
                     match &self.pipelines[index] {
                         PipelineInfo::Rendering { vert, frag } => {
-                            let vert_artifact = shader_module::create_shader_module(
-                                vert,
-                                shaderc::ShaderKind::Vertex,
-                                shader_compiler,
-                            )?;
-                            let vert =
-                                ShaderCreateInfo::new(vert_artifact.as_binary(), &vert.entry_point);
+                            let vert_artifact = shader_compiler
+                                .create_shader_module(vert, shaderc::ShaderKind::Vertex)?;
+                            let vert = ShaderCreateInfo::new(&vert_artifact, &vert.entry_point);
 
-                            let frag_arifact = shader_module::create_shader_module(
-                                frag,
-                                shaderc::ShaderKind::Fragment,
-                                shader_compiler,
-                            )?;
-                            let frag =
-                                ShaderCreateInfo::new(frag_arifact.as_binary(), &frag.entry_point);
+                            let frag_arifact = shader_compiler
+                                .create_shader_module(frag, shaderc::ShaderKind::Fragment)?;
+                            let frag = ShaderCreateInfo::new(&frag_arifact, &frag.entry_point);
 
                             match self.kind.as_mut().unwrap() {
                                 Backend::Ash(ash) => {
@@ -164,13 +148,9 @@ impl<'a> RenderBundleStatic<'a> {
                         }
 
                         PipelineInfo::Compute { comp } => {
-                            let comp_artifact = shader_module::create_shader_module(
-                                comp,
-                                shaderc::ShaderKind::Compute,
-                                shader_compiler,
-                            )?;
-                            let comp =
-                                ShaderCreateInfo::new(comp_artifact.as_binary(), &comp.entry_point);
+                            let comp_artifact = shader_compiler
+                                .create_shader_module(comp, shaderc::ShaderKind::Compute)?;
+                            let comp = ShaderCreateInfo::new(&comp_artifact, &comp.entry_point);
 
                             match self.kind.as_mut().unwrap() {
                                 Backend::Ash(ash) => ash.rebuild_compute_pipeline(index, comp)?,
@@ -204,7 +184,7 @@ impl<'a> RenderBundleStatic<'a> {
     pub fn switch(
         &mut self,
         window: &impl HasRawWindowHandle,
-        shader_compiler: &mut Compiler,
+        shader_compiler: &mut ShaderCompiler,
     ) -> Result<()> {
         puffin::profile_function!();
         self.wait_idle();
@@ -233,19 +213,13 @@ impl<'a> RenderBundleStatic<'a> {
         for pipeline in &self.pipelines {
             match pipeline {
                 PipelineInfo::Rendering { vert, frag } => {
-                    let vert_artifact = shader_module::create_shader_module(
-                        vert,
-                        shaderc::ShaderKind::Vertex,
-                        shader_compiler,
-                    )?;
-                    let vert = ShaderCreateInfo::new(vert_artifact.as_binary(), &vert.entry_point);
+                    let vert_artifact =
+                        shader_compiler.create_shader_module(vert, shaderc::ShaderKind::Vertex)?;
+                    let vert = ShaderCreateInfo::new(&vert_artifact, &vert.entry_point);
 
-                    let frag_arifact = shader_module::create_shader_module(
-                        frag,
-                        shaderc::ShaderKind::Fragment,
-                        shader_compiler,
-                    )?;
-                    let frag = ShaderCreateInfo::new(frag_arifact.as_binary(), &frag.entry_point);
+                    let frag_arifact = shader_compiler
+                        .create_shader_module(frag, shaderc::ShaderKind::Fragment)?;
+                    let frag = ShaderCreateInfo::new(&frag_arifact, &frag.entry_point);
 
                     match self.kind.as_mut().unwrap() {
                         Backend::Ash(ash) => ash.push_render_pipeline(vert, frag)?,
@@ -253,12 +227,9 @@ impl<'a> RenderBundleStatic<'a> {
                     }
                 }
                 PipelineInfo::Compute { comp } => {
-                    let comp_artifact = shader_module::create_shader_module(
-                        comp,
-                        shaderc::ShaderKind::Compute,
-                        shader_compiler,
-                    )?;
-                    let comp = ShaderCreateInfo::new(comp_artifact.as_binary(), &comp.entry_point);
+                    let comp_artifact =
+                        shader_compiler.create_shader_module(comp, shaderc::ShaderKind::Compute)?;
+                    let comp = ShaderCreateInfo::new(&comp_artifact, &comp.entry_point);
                     match self.kind.as_mut().unwrap() {
                         Backend::Ash(ash) => ash.push_compute_pipeline(comp)?,
                         Backend::Wgpu(wgpu) => wgpu.push_compute_pipeline(comp)?,
@@ -291,7 +262,7 @@ impl Renderer for RenderBundleStatic<'_> {
         self.wh = (width, height);
         self.get_active_mut().resize(width, height)
     }
-    fn render(&mut self, push_constant: &[u8]) -> Result<()> {
+    fn render(&mut self, push_constant: PushConstant) -> Result<()> {
         puffin::profile_function!();
         self.get_active_mut().render(push_constant)
     }
@@ -325,7 +296,7 @@ impl Renderer for AshRender<'_> {
         Ok(self.resize()?)
     }
 
-    fn render(&mut self, push_constant: &[u8]) -> Result<()> {
+    fn render(&mut self, push_constant: PushConstant) -> Result<()> {
         Ok(self.render(push_constant)?)
     }
 
@@ -358,7 +329,7 @@ impl Renderer for pilka_wgpu::WgpuRender {
         Ok(())
     }
 
-    fn render(&mut self, push_constant: &[u8]) -> Result<()> {
+    fn render(&mut self, push_constant: PushConstant) -> Result<()> {
         Ok(Self::render(self, push_constant)?)
     }
 
